// 
// IR REMOTE TRANSMITTER PIC12F1938 VERSION
//	ver1.1  Jun.30th,2013 CyberPit HILO
//	
// Note:
//		This is Main Function XC8 C Language Source Code
//		Also need to link with "transmit.as" Assembler Source
//		which was inspired by following people
//			http://www.interline.or.jp/%7Etomcat/index2.html
//			http://phys.sci.hokudai.ac.jp/LABS/yts/tsujimi/tsujimi.html
//		Regardsing them, I also keep this code as freeware for the DIY freaks.
//												Thank you!	
//

#include <pic.h>
#include<pic16f1936.h>
#include <htc.h>
#include <xc.h>

#define  _XTAL_FREQ 4000000 //4 MHz


// first CONFIG reg
__CONFIG( 
    FOSC_INTOSC & WDTE_OFF & PWRTE_ON & MCLRE_OFF & CP_OFF & CPD_OFF & BOREN_OFF  &
    CLKOUTEN_OFF & IESO_OFF & FCMEN_OFF);
// second CONFIG reg
__CONFIG(WRT_OFF & PLLEN_OFF & STVREN_ON &  LVP_OFF);

unsigned char tx_data[4];
unsigned char ScnStatus;//40 kHz: set 0th bit to 1
						//33 kHz: set 1st bit to 1
						//38 kHz: clear 0th and 1st bits to 0
						//sharp mode: set 2nd bit to 1
unsigned char rmcbyt;	// Bytes for sending data
unsigned char rmcbit;	// Additionl bits for sending data
unsigned char start_h;	// leader on time
unsigned char start_l;	// leader off time
unsigned char data_0h;	// data 0 on time
unsigned char data_1h;	// data 1 on time
unsigned char data_0l;	// data 0 off time
unsigned char data_1l;	// data 1 off time
unsigned char stop_h;	// stop on time
unsigned char rptwait_bf;	//repeat wait time for sharp mode
unsigned char rptwait;	// repeat wait time for sharp mode
unsigned char read_key;	// Key read buffer	


extern void transmit(void);// assembly routine 

/*
----< ALPINE IR FORMAT SETUP >----
*/
void Alpine4byte(void){
	ScnStatus=0x00;//38kHz: Toshiba mode@(Each@Value*0.1mSec)
	rmcbyt=4;		//f[^̑oCg	
	rmcbit=0; 		//[rbg
	start_h=90; 	//X^[gM  ---- A
	start_l=42; 	//X^[gM   --- B
	data_0h=5; 		//f[^OM   --- C
	data_0l=6; 		//f[^OM   --- E
	data_1h=5; 		//f[^PM   --- D
	data_1l=17; 	//f[^PM   --- F
	stop_h=7;		//XgbvM   --- G
	rptwait_bf=0; 
	rptwait=0;		//s[g҂ (1 = 10ms)- H
}

/*
----< PIONEER IR FORMAT SEETUP >----
*/
void Carrozzeria4byte(void){
	ScnStatus=0x01;//40kHz:
	rmcbyt=4;		//f[^̑oCg	
	rmcbit=0; 		//[rbg
	start_h=84;  	//X^[gM  ---- A
	start_l=43;  	//X^[gM   --- B
	data_0h=5; 		//f[^OM   --- C
	data_0l=5; 		//f[^OM   --- E
	data_1h=5;	 	//f[^PM   --- D
	data_1l=16; 	//f[^PM   --- F
	stop_h=7;		//XgbvM   --- G
	rptwait_bf=0; 
	rptwait=9;		//s[g҂ (1 = 10ms)- H
}

/*
----< READING KEY MATRIX AND TRANSMIT IR >----
*/
void KeySense(void)
{
RC6 = 0;	// To read Slide switch
if (RB5 == 0)
 {
 // ========================================================================
 //		 			Slide Switch is mode II (TV mode)
 // ========================================================================
 RC6 = 1; // To Keep Battery Safe
 RB7 = 0;				// Select Upper (Even) Group Keys
 RB6 = 1;				// UnSelect Lower(Odd) Group Keys
 read_key =	(PORTB & 0b00011111);	// Capture Even Numbered keys(Pushed == L)
 switch(read_key)
	{
	case	0b00011110:
				// (K2) eonon CH+
				Alpine4byte();	//as set kaseikyo 38kHz mode
				tx_data[0]=0x02;
				tx_data[1]=0xFD;
				tx_data[2]=0x0F;
				tx_data[3]=0xF0;
			transmit();		//IR transmition	
			while(RB0==0);	//wait for key release
			IOCBF=0;	//interruption enable again
			break;

	case	0b00011101:
				// (K4)eonon SCAN
				Alpine4byte();	//as set kaseikyo 38kHz mode
				tx_data[0]=0x02;
				tx_data[1]=0xFD;
				tx_data[2]=0x17;
				tx_data[3]=0xE8;
			transmit();		//IR transmition	
			while(RB1==0);	//wait for key release
			IOCBF=0;	//interruption enable again
			break;

	case	0b00011011:
				// (K6)eonon VIDEO
				Alpine4byte();	//as set kaseikyo 38kHz mode
				tx_data[0]=0x02;
				tx_data[1]=0xFD;
				tx_data[2]=0x16;
				tx_data[3]=0xE9;
			transmit();				//1st half Only IR transmition
			while(RB2==0);	//wait for key release
			IOCBF=0;	//interruption enable again
			break;

	case	0b00010111:
				// (K8)Pioneer VOLUME-DOWN
				 Carrozzeria4byte();	//as set toshiba mode
				tx_data[0]=0xAD;
				tx_data[1]=0x52;
				tx_data[2]=0x0B;
				tx_data[3]=0xF4;
			transmit();				//1st half Only IR transmition
			while(RB3==0);	//wait for key release
			IOCBF=0;	//interruption enable again
			break;

	case	0b00001111:
				// (K10)  LCD MONITOR SOURCE SELECT
				Alpine4byte();	//as set toshiba mode
				tx_data[0]=0x86;
				tx_data[1]=0x6B;
				tx_data[2]=0x1B;
				tx_data[3]=0xE4;
				transmit();			//IR transmition
			while(RB4==0);	//wait for key release
			IOCBF=0;	//interruption enable again
	default:
			break;
		}

RB7 = 1;		// UnSelect Upper (Even) Group Keys
RB6 = 0;		// Select Lower(Odd) Group Keys
read_key =	(PORTB & 0b00011111);	// Capture Odd Numbered keys (Pushed == L)
switch(read_key)
	{
	case	0b00011110:
				// (K1) eonon CH-
				Alpine4byte();	//as set kaseikyo 38kHz mode
				tx_data[0]=0x02;
				tx_data[1]=0xFD;
				tx_data[2]=0x13;
				tx_data[3]=0xEC;
			transmit();		//IR transmition	
			while(RB0==0);	//wait for key release
			IOCBF=0;	//interruption enable again
			break;

	case	0b00011101:
				// (K3) eonon EPG
				Alpine4byte();	//as set kaseikyo 38kHz mode
				tx_data[0]=0x02;
				tx_data[1]=0xFD;
				tx_data[2]=0x01;
				tx_data[3]=0xFE;
			transmit();		//IR transmition	
			while(RB1==0);	//wait for key release
			IOCBF=0;	//interruption enable again
			break;

	case	0b00011011:
				// (K5)eonon SUB TEXT ON/OFF
				Alpine4byte();	//as set kaseikyo 38kHz mode
				tx_data[0]=0x02;
				tx_data[1]=0xFD;
				tx_data[2]=0x1B;
				tx_data[3]=0xE4  ;
			transmit();				//1st half Only IR transmition
			while(RB2==0);	//wait for key release
			IOCBF=0;	//interruption enable again
			break;

	case	0b00010111:
				// (K7) Pioneer VOLUME-UP
				 Carrozzeria4byte();	//as set toshiba mode
				tx_data[0]=0xAD;
				tx_data[1]=0x52;
				tx_data[2]=0x0A;
				tx_data[3]=0xF5  ;
			transmit();				//1st half Only IR transmition
			while(RB3==0);	//wait for key release
			IOCBF=0;	//interruption enable again
			break;

	case	0b00001111:
				// (K9) LCD MONITOR SOURCE
				Alpine4byte();	//as set toshiba mode
				tx_data[0]=0x86;
				tx_data[1]=0x6B;
				tx_data[2]=0x1B;
				tx_data[3]=0xE4;
				transmit();			//IR transmition
			while(RB4==0);	//wait for key release
			IOCBF=0;	//interruption enable again
	default:
			break;
		}
 }
else
 {
 // ========================================================================
 // 				Slide Swithch is mode I (DVD mode)
 // ========================================================================
 RC6 = 1; // To Keep Battery Safe
 RB7 = 0;				// Select Upper (Even) Group Keys
 RB6 = 1;				// UnSelect Lower(Odd) Group Keys
 read_key =	(PORTB & 0b00011111);	// Capture Even Numbered keys(Pushed == L)
 switch(read_key)
	{
	case	0b00011110:
				// (K2)  Pioneer DISC NEXT TRK
				 Carrozzeria4byte();	//as set toshiba mode
				tx_data[0]=0xAD;
				tx_data[1]=0x52;
				tx_data[2]=0x99;
				tx_data[3]=0x66;
			transmit();				//1st half IR transmition
 				__delay_us(25640);	// insert interval
				tx_data[0]=0xaf;
				tx_data[1]=0x50;
				tx_data[2]=0x71;
				tx_data[3]=0x8E;
			transmit();		//2nd half IR transmition
			while(RB0==0);	//wait for key release
			IOCBF=0;	//interruption enable again
			break;

	case	0b00011101:
				// (K4) Pioneer DISC FWD
				 Carrozzeria4byte();	//as set toshiba mode
				tx_data[0]=0xAD;
				tx_data[1]=0x52;
				tx_data[2]=0x99;
				tx_data[3]=0x66;
			transmit();				//1st half IR transmition
 				__delay_us(25640);	// insert interval
				tx_data[0]=0xaf;
				tx_data[1]=0x50;
				tx_data[2]=0xEB;
				tx_data[3]=0x14;
			transmit();		//2nd half IR transmition
//			while(RB1==0);	//wait for key release
			IOCBF=0;	//interruption enable again
			break;

	case	0b00011011:
				// (K6)Pioneer SOURCE
				 Carrozzeria4byte();	//as set toshiba mode
				tx_data[0]=0xAD;
				tx_data[1]=0x52;
				tx_data[2]=0x1A;
				tx_data[3]=0xE5;
			transmit();				//1st half Only IR transmition
//			while(RB2==0);	//wait for key release
			__delay_ms(17);		// To adjust repeat gap perid
			IOCBF=0;	//interruption enable again
			break;

	case	0b00010111:
				// (K8)Pioneer VOLUME-DOWN
				 Carrozzeria4byte();	//as set toshiba mode
				tx_data[0]=0xAD;
				tx_data[1]=0x52;
				tx_data[2]=0x0B;
				tx_data[3]=0xF4;
			transmit();				//1st half Only IR transmition
//			while(RB3==0);	//wait for key release
			__delay_ms(17);		// To adjust repeat gap perid
			IOCBF=0;	//interruption enable again
			break;

	case	0b00001111:
				// (K10)  LCD MONITOR ON/OFF
				Alpine4byte();	//as set toshiba mode
				tx_data[0]=0x86;
				tx_data[1]=0x6B;
				tx_data[2]=0x12;
				tx_data[3]=0xED;
				transmit();			//IR transmition
			while(RB4==0);	//wait for key release
			IOCBF=0;	//interruption enable again
	default:
			break;
		}

RB7 = 1;		// UnSelect Upper (Even) Group Keys
RB6 = 0;		// Select Lower(Odd) Group Keys
read_key =	(PORTB & 0b00011111);	// Capture Odd Numbered keys (Pushed == L)
switch(read_key)
	{
	case	0b00011110:
				// (K1) Pioneer DISC PRIV TRK
				 Carrozzeria4byte();	//as set toshiba mode
				tx_data[0]=0xAD;
				tx_data[1]=0x52;
				tx_data[2]=0x99;
				tx_data[3]=0x66;
			transmit();				//1st half IR transmition
 				__delay_us(25640);	// insert interval
				tx_data[0]=0xaf;
				tx_data[1]=0x50;
				tx_data[2]=0x72;
				tx_data[3]=0x8D;
			transmit();		//2nd half IR transmition
			while(RB0==0);	//wait for key release
			IOCBF=0;	//interruption enable again
			break;

	case	0b00011101:
				// (K3) Pioneer DISC FWD
				 Carrozzeria4byte();	//as set toshiba mode
				tx_data[0]=0xAD;
				tx_data[1]=0x52;
				tx_data[2]=0x99;
				tx_data[3]=0x66;
			transmit();				//1st half IR transmition
 				__delay_us(25640);	// insert interval
				tx_data[0]=0xaf;
				tx_data[1]=0x50;
				tx_data[2]=0x79;
				tx_data[3]=0x86;
			transmit();		//2nd half IR transmition
//			while(RB1==0);	//wait for key release
			IOCBF=0;	//interruption enable again
			break;

	case	0b00011011:
				// (K5) Pioneer DISC PLAY/PAUSE
				 Carrozzeria4byte();	//as set toshiba mode
				tx_data[0]=0xAD;
				tx_data[1]=0x52;
				tx_data[2]=0x99;
				tx_data[3]=0x66;
			transmit();				//1st half IR transmition
 				__delay_us(25640);	// insert interval
				tx_data[0]=0xaf;
				tx_data[1]=0x50;
				tx_data[2]=0x73;
				tx_data[3]=0x8C;
			transmit();		//2nd half IR transmition
			while(RB2==0);	//wait for key release
			IOCBF=0;	//interruption enable again
			break;

	case	0b00010111:
				// (K7) Pioneer VOLUME-UP
				 Carrozzeria4byte();	//as set toshiba mode
				tx_data[0]=0xAD;
				tx_data[1]=0x52;
				tx_data[2]=0x0A;
				tx_data[3]=0xF5  ;
			transmit();				//1st half Only IR transmition
			while(RB3==0);	//wait for key release
			IOCBF=0;	//interruption enable again
			break;

	case	0b00001111:
				// (K9) LCD MONITOR ON/OFF
				Alpine4byte();	//as set toshiba mode
				tx_data[0]=0x86;
				tx_data[1]=0x6B;
				tx_data[2]=0x12;
				tx_data[3]=0xED;
				transmit();			//IR transmition
			while(RB4==0);	//wait for key release
			IOCBF=0;	//interruption enable again
	default:
			break;
		}
 }
RB7 = RB6 =0;	// Prepare for the any Key Press Detectable
RC6 = 1;		// For save battery
}


/*
=====<< THE MAIN FUNCTION >>====
*/
void main (void)
{
 OSCCON	=	0b01101010;	//4 MHz Internal RC
 /*Port-A o͐ݒ*/	
 ANSELA = 	0b00110000;	// RA4/5 are CPS, others are Digital Ports
 TRISA 	= 	0b00110000;	// Define RA4/5 is CPS port as trisate/Input (==1), others are outputs
 /*Port-B o͐ݒ*/	
 ANSELB = 	0b00000000;	// Define RBx are all Digital(==0) 
 TRISB 	= 	0b00111111;	// Define RB0/1/2/3/4/5 is trisate/Inputs (==1)
 nWPUEN = 0;			// Enable Weak Pull-up Port B pins
 WPUB 	=  	0b00111111;	// Weak pull-up from RB0 to RB5 (==1)
 IOCBN	=	0b00111111;	// Enable Fall-Edge change Interrupt for RB0-RB4 input pins 
 IOCBP	=	0b00000000;	// Disable Rise-Edge Interrupt detection
 IOCBF	=	0b00000000;	// Clear interruption flag
 PORTB	= 	0b00111111;
 /*Port-C o͐ݒ*/	
 TRISC 	= 	0b00111111;	// Define RC7 for LED, RC6 for Slide SW Scan, others are Inputs
// LATC 	=	0b01111111;`// Same meaning as write to PORTC
 PORTC	= 	0b01111111;

 Alpine4byte();	// Inital format set as a toshiba mode

// RC7 =	1; 				// Turn on LED
//	__delay_ms(500);	// LED TEST FLASHING
 RC7 = 0;		// Turn off IR LED
 RC6 = 1;		// Set High to Inactive current save.
 RB7 = RB6 =0;	// Prepare for the any Key Press Detectable

 while(1)
	{
	IOCBF =0; 	// Clear Enable Interrupt-On-Change Flag
	IOCIE =1;	// Enabke Interrupt-On-Change 
	GIE=0;		// As a real interrupt shold not be occured, If GIE had set wake-up from s1leep not works well. 
	if ((PORTB & 0b00011111) == 0b00011111)	// Check if NO key pressed
		SLEEP();							// Enter Low power sleep mode
	__delay_ms(7);		// Removal of chattering and prepelation time for Combined Push
	KeySense();			// Read key and transmit IR code if much the key pattern
//	__delay_ms(110);	// Removal of release chattering
	}
}


/*
----<Any Inturrupt Function >----
*/

static void interrupt ISR(void)
{
return;
}